@@ -1,5 +1,5 @@
 //
-// Copyright (C) 2013 Codership Oy <info@codership.com>
+// Copyright (C) 2013-2018 Codership Oy <info@codership.com>
 //
 
 #ifndef GALERA_KEY_DATA_HPP
@@ -7,6 +7,8 @@
 
 #include "wsrep_api.h"
 
+#include <ostream>
+
 namespace galera
 {
 
@@ -37,12 +39,21 @@ struct KeyData
 
     bool shared() const { return type == WSREP_KEY_SHARED; }
 
+    void print(std::ostream& os) const;
+
 private:
 
     KeyData& operator = (const KeyData&);
 
 }; /* struct KeyData */
 
+inline std::ostream&
+operator << (std::ostream& os, const KeyData& kd)
+{
+    kd.print(os);
+    return os;
+}
+
 } /* namespace galera */
 
 #endif /* GALERA_KEY_DATA_HPP */
