@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013 Codership Oy <info@codership.com>
+ * Copyright (C) 2013-2016 Codership Oy <info@codership.com>
  *
  * $Id$
  */
@@ -14,10 +14,11 @@ gu_init (gu_log_cb_t log_cb)
 {
     gu_conf_set_log_callback (log_cb);
 
-    if (gu_page_size() != GU_PAGE_SIZE)
+    /* this is needed in gu::MMap::sync() */
+    size_t const page_size = GU_PAGE_SIZE;
+    if (page_size & (page_size - 1))
     {
-        gu_fatal("GU_PAGE_SIZE(%u) does not maptch real system page size(%zu)",
-                 GU_PAGE_SIZE, gu_page_size());
+        gu_fatal("GU_PAGE_SIZE(%z) is not a power of 2", GU_PAGE_SIZE);
         gu_abort();
     }
 
