@@ -10,6 +10,8 @@
 
 using namespace gcache;
 
+#define TEST_CACHE "test.cache"
+
 int
 main (int argc, char* argv[])
 {
@@ -25,7 +27,7 @@ main (int argc, char* argv[])
     if (argc > 1) fname.assign(argv[1]); // take supplied file name if any
     gu::Config conf;
     GCache::register_params(conf);
-    conf.parse("gcache.name = test.cache; gcache.size = 16K");
+    conf.parse("gcache.name = " TEST_CACHE "; gcache.size = 16K");
     GCache* cache = new GCache (conf, "");
 
     log_info  << "";
@@ -33,6 +35,7 @@ main (int argc, char* argv[])
     log_info  << "";
 
     delete cache;
+    ::unlink(TEST_CACHE);
 
     log_info  << "Exit: " << ret;
 
