@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2018 Codership Oy <info@codership.com>
+ * Copyright (C) 2010-2020 Codership Oy <info@codership.com>
  *
  * $Id$
  */
@@ -26,35 +26,35 @@ START_TEST(test1)
 
     gcache::PageStore ps (dir_name, keep_size, page_size, 0, false);
 
-    fail_if(ps.count()       != 0,"expected count 0, got %zu",ps.count());
-    fail_if(ps.total_pages() != 0,"expected 0 pages, got %zu",ps.total_pages());
-    fail_if(ps.total_size()  != 0,"expected size 0, got %zu", ps.total_size());
+    ck_assert_msg(ps.count()       == 0,"expected count 0, got %zu",ps.count());
+    ck_assert_msg(ps.total_pages() == 0,"expected 0 pages, got %zu",ps.total_pages());
+    ck_assert_msg(ps.total_size()  == 0,"expected size 0, got %zu", ps.total_size());
 
     void* buf = ps.malloc (3 + bh_size);
 
-    fail_if (0 == buf);
-    fail_if(ps.count()       != 1,"expected count 1, got %zu",ps.count());
-    fail_if(ps.total_pages() != 1,"expected 1 pages, got %zu",ps.total_pages());
+    ck_assert(0 != buf);
+    ck_assert_msg(ps.count()       == 1,"expected count 1, got %zu",ps.count());
+    ck_assert_msg(ps.total_pages() == 1,"expected 1 pages, got %zu",ps.total_pages());
 
     void* tmp = ps.realloc (buf, 2 + bh_size);
 
-    fail_if (buf != tmp);
-    fail_if(ps.count()       != 1,"expected count 1, got %zu",ps.count());
-    fail_if(ps.total_pages() != 1,"expected 1 pages, got %zu",ps.total_pages());
+    ck_assert(buf == tmp);
+    ck_assert_msg(ps.count()       == 1,"expected count 1, got %zu",ps.count());
+    ck_assert_msg(ps.total_pages() == 1,"expected 1 pages, got %zu",ps.total_pages());
 
     tmp = ps.realloc (buf, 4 + bh_size); // here new page should be allocated
 
-    fail_if (0 == tmp);
-    fail_if (buf == tmp);
-    fail_if(ps.count()       != 2,"expected count 2, got %zu",ps.count());
-    fail_if(ps.total_pages() != 1,"expected 1 pages, got %zu",ps.total_pages());
+    ck_assert(0   != tmp);
+    ck_assert(buf != tmp);
+    ck_assert_msg(ps.count()       == 2,"expected count 2, got %zu",ps.count());
+    ck_assert_msg(ps.total_pages() == 1,"expected 1 pages, got %zu",ps.total_pages());
 
     ps_free(tmp);
     ps.discard (ptr2BH(tmp));
 
-    fail_if(ps.count()       != 2,"expected count 2, got %zu",ps.count());
-    fail_if(ps.total_pages() != 0,"expected 0 pages, got %zu",ps.total_pages());
-    fail_if(ps.total_size()  != 0,"expected size 0, got %zu", ps.total_size());
+    ck_assert_msg(ps.count()       == 2,"expected count 2, got %zu",ps.count());
+    ck_assert_msg(ps.total_pages() == 0,"expected 0 pages, got %zu",ps.total_pages());
+    ck_assert_msg(ps.total_size()  == 0,"expected size 0, got %zu", ps.total_size());
 }
 END_TEST
 
@@ -71,7 +71,7 @@ START_TEST(test2)
 
     uint8_t* buf = static_cast<uint8_t*>(ps.malloc (page_size));
 
-    fail_if (0 == buf);
+    ck_assert(0 != buf);
 
     while (--page_size)
     {
@@ -98,17 +98,17 @@ START_TEST(test3) // check that all page size is efficiently used
     ssize_t ptr_size = (page_size / 2);
 
     void* ptr1 = ps.malloc (ptr_size);
-    fail_if (0 == ptr1);
+    ck_assert(0 != ptr1);
 
     void* ptr2 = ps.malloc (ptr_size);
-    fail_if (0 == ptr2);
+    ck_assert(0 != ptr2);
 
-    fail_if (ps.count() != 1, "ps.count() = %zd, expected 1", ps.count());
+    ck_assert_msg(ps.count() == 1, "ps.count() = %zd, expected 1", ps.count());
 
     // check that ptr2 is adjacent to ptr1
     void* tmp = static_cast<uint8_t*>(ptr1) + ptr_size;
 
-    fail_if (tmp != ptr2, "tmp = %p, ptr2 = %p", tmp, ptr2);
+    ck_assert_msg(tmp == ptr2, "tmp = %p, ptr2 = %p", tmp, ptr2);
 
     ps_free(ptr1); ps.discard(ptr2BH(ptr1));
     ps_free(ptr2); ps.discard(ptr2BH(ptr2));
