@@ -93,7 +93,7 @@ START_TEST(test_message)
 
     UserMessage um(0, uuid1, view_id, seq, aru_seq, seq_range, O_SAFE, 75433, 0xab,
                    Message::F_SOURCE);
-    fail_unless(um.serial_size() % 4 == 0);
+    ck_assert(um.serial_size() % 4 == 0);
     check_serialization(um, um.serial_size(), UserMessage());
 
     AggregateMessage am(0xab, 17457, 0x79);
@@ -140,7 +140,8 @@ START_TEST(test_input_map_insert)
     try
     {
         im.insert(0, UserMessage(0, uuid1, view, 0));
-        fail("");
+        ck_abort_msg("Exception not thrown, input map has not been "
+                     "reset/initialized yet");
     }
     catch (...)
     {  }
@@ -162,14 +163,14 @@ START_TEST(test_input_map_insert)
     for (seqno_t s = 0; s < 10; ++s)
     {
         InputMap::iterator i = im.find(0, s);
-        fail_if(i == im.end());
-        fail_unless(InputMapMsgIndex::value(i).msg().source() == uuid1);
-        fail_unless(InputMapMsgIndex::value(i).msg().seq() == s);
+        ck_assert(i != im.end());
+        ck_assert(InputMapMsgIndex::value(i).msg().source() == uuid1);
+        ck_assert(InputMapMsgIndex::value(i).msg().seq() == s);
 
         i = im.find(1, s);
-        fail_if(i == im.end());
-        fail_unless(InputMapMsgIndex::value(i).msg().source() == uuid2);
-        fail_unless(InputMapMsgIndex::value(i).msg().seq() == s);
+        ck_assert(i != im.end());
+        ck_assert(InputMapMsgIndex::value(i).msg().source() == uuid2);
+        ck_assert(InputMapMsgIndex::value(i).msg().seq() == s);
     }
 
 }
@@ -186,21 +187,21 @@ START_TEST(test_input_map_find)
 
     im.insert(0, UserMessage(0, uuid1, view, 0));
 
-    fail_if(im.find(0, 0) == im.end());
+    ck_assert(im.find(0, 0) != im.end());
 
 
     im.insert(0, UserMessage(0, uuid1, view, 2));
     im.insert(0, UserMessage(0, uuid1, view, 4));
     im.insert(0, UserMessage(0, uuid1, view, 7));
 
-    fail_if(im.find(0, 2) == im.end());
-    fail_if(im.find(0, 4) == im.end());
-    fail_if(im.find(0, 7) == im.end());
+    ck_assert(im.find(0, 2) != im.end());
+    ck_assert(im.find(0, 4) != im.end());
+    ck_assert(im.find(0, 7) != im.end());
 
-    fail_unless(im.find(0, 3) == im.end());
-    fail_unless(im.find(0, 5) == im.end());
-    fail_unless(im.find(0, 6) == im.end());
-    fail_unless(im.find(0, 8) == im.end());
+    ck_assert(im.find(0, 3) == im.end());
+    ck_assert(im.find(0, 5) == im.end());
+    ck_assert(im.find(0, 6) == im.end());
+    ck_assert(im.find(0, 8) == im.end());
 }
 END_TEST
 
@@ -215,34 +216,34 @@ START_TEST(test_input_map_safety)
     im.reset(1);
 
     im.insert(index1, UserMessage(0, uuid1, view, 0));
-    fail_unless(im.aru_seq() == 0);
+    ck_assert(im.aru_seq() == 0);
     im.insert(index1, UserMessage(0, uuid1, view, 1));
-    fail_unless(im.aru_seq() == 1);
+    ck_assert(im.aru_seq() == 1);
     im.insert(index1, UserMessage(0, uuid1, view, 2));
-    fail_unless(im.aru_seq() == 2);
+    ck_assert(im.aru_seq() == 2);
     im.insert(index1, UserMessage(0, uuid1, view, 3));
-    fail_unless(im.aru_seq() == 3);
+    ck_assert(im.aru_seq() == 3);
     im.insert(index1, UserMessage(0, uuid1, view, 5));
-    fail_unless(im.aru_seq() == 3);
+    ck_assert(im.aru_seq() == 3);
 
     im.insert(index1, UserMessage(0, uuid1, view, 4));
-    fail_unless(im.aru_seq() == 5);
+    ck_assert(im.aru_seq() == 5);
 
     InputMap::iterator i = im.find(index1, 0);
-    fail_unless(im.is_fifo(i) == true);
-    fail_unless(im.is_agreed(i) == true);
-    fail_if(im.is_safe(i) == true);
+    ck_assert(im.is_fifo(i) == true);
+    ck_assert(im.is_agreed(i) == true);
+    ck_assert(im.is_safe(i) == false);
     im.set_safe_seq(index1, 0);
-    fail_unless(im.is_safe(i) == true);
+    ck_assert(im.is_safe(i) == true);
 
     im.set_safe_seq(index1, 5);
     i = im.find(index1, 5);
-    fail_unless(im.is_safe(i) == true);
+    ck_assert(im.is_safe(i) == true);
 
     im.insert(index1, UserMessage(0, uuid1, view, 7));
     im.set_safe_seq(index1, im.aru_seq());
     i = im.find(index1, 7);
-    fail_if(im.is_safe(i) == true);
+    ck_assert(im.is_safe(i) == false);
 
 }
 END_TEST
@@ -265,17 +266,18 @@ START_TEST(test_input_map_erase)
     for (seqno_t s = 0; s < 10; ++s)
     {
         InputMap::iterator i = im.find(index1, s);
-        fail_unless(i != im.end());
+        ck_assert(i != im.end());
         im.erase(i);
         i = im.find(index1, s);
-        fail_unless(i == im.end());
+        ck_assert(i == im.end());
         (void)im.recover(index1, s);
     }
     im.set_safe_seq(index1, 9);
     try
     {
         im.recover(index1, 9);
-        fail("");
+        ck_abort_msg("Exception not thrown, "
+                     "setting safe seq should purge index");
     }
     catch (...) { }
 }
@@ -377,23 +379,23 @@ START_TEST(test_input_map_random_insert)
     for (InputMap::iterator i = im.begin(); i != im.end(); ++i)
     {
         const InputMapMsg& msg(InputMapMsgIndex::value(i));
-        fail_unless(msg.msg() == msgs[n].second);
-        fail_if(im.is_safe(i) == true);
+        ck_assert(msg.msg() == msgs[n].second);
+        ck_assert(im.is_safe(i) == false);
         ++n;
     }
 
-    fail_unless(im.aru_seq() == n_seqnos - 1);
-    fail_unless(im.safe_seq() == -1);
+    ck_assert(im.aru_seq() == n_seqnos - 1);
+    ck_assert(im.safe_seq() == -1);
 
     for (size_t i = 0; i < n_uuids; ++i)
     {
-        fail_unless(im.range(i) ==
+        ck_assert(im.range(i) ==
                     Range(n_seqnos,
                           n_seqnos - 1));
 
         im.set_safe_seq(i, n_seqnos - 1);
     }
-    fail_unless(im.safe_seq() == n_seqnos - 1);
+    ck_assert(im.safe_seq() == n_seqnos - 1);
 
 }
 END_TEST
@@ -409,47 +411,47 @@ START_TEST(test_input_map_gap_range_list)
 
     std::vector<gcomm::evs::Range> gap_range(
         im.gap_range_list(0, gcomm::evs::Range(0, 2)));
-    fail_unless(gap_range.size() == 1);
-    fail_unless(gap_range.begin()->lu() == 1);
-    fail_unless(gap_range.begin()->hs() == 1);
+    ck_assert(gap_range.size() == 1);
+    ck_assert(gap_range.begin()->lu() == 1);
+    ck_assert(gap_range.begin()->hs() == 1);
 
     im.insert(0, gcomm::evs::UserMessage(0, uuid, view_id, 4, 0));
     gap_range = im.gap_range_list(0, gcomm::evs::Range(0, 4));
-    fail_unless(gap_range.size() == 2);
-    fail_unless(gap_range.begin()->lu() == 1);
-    fail_unless(gap_range.begin()->hs() == 1);
-    fail_unless(gap_range.rbegin()->lu() == 3);
-    fail_unless(gap_range.rbegin()->hs() == 3);
+    ck_assert(gap_range.size() == 2);
+    ck_assert(gap_range.begin()->lu() == 1);
+    ck_assert(gap_range.begin()->hs() == 1);
+    ck_assert(gap_range.rbegin()->lu() == 3);
+    ck_assert(gap_range.rbegin()->hs() == 3);
 
     // Although there are two messages missing, limiting the range to 0,2
     // should return only the first one.
     gap_range = im.gap_range_list(0, gcomm::evs::Range(0, 2));
-    fail_unless(gap_range.size() == 1);
-    fail_unless(gap_range.begin()->lu() == 1);
-    fail_unless(gap_range.begin()->hs() == 1);
+    ck_assert(gap_range.size() == 1);
+    ck_assert(gap_range.begin()->lu() == 1);
+    ck_assert(gap_range.begin()->hs() == 1);
 
     im.insert(0, gcomm::evs::UserMessage(0, uuid, view_id, 8, 0));
     gap_range = im.gap_range_list(0, gcomm::evs::Range(0, 8));
-    fail_unless(gap_range.size() == 3);
-    fail_unless(gap_range.begin()->lu() == 1);
-    fail_unless(gap_range.begin()->hs() == 1);
-    fail_unless(gap_range.rbegin()->lu() == 5);
-    fail_unless(gap_range.rbegin()->hs() == 7);
+    ck_assert(gap_range.size() == 3);
+    ck_assert(gap_range.begin()->lu() == 1);
+    ck_assert(gap_range.begin()->hs() == 1);
+    ck_assert(gap_range.rbegin()->lu() == 5);
+    ck_assert(gap_range.rbegin()->hs() == 7);
 
     im.insert(0, gcomm::evs::UserMessage(0, uuid, view_id, 3, 0));
     gap_range = im.gap_range_list(0, gcomm::evs::Range(0, 8));
-    fail_unless(gap_range.size() == 2);
-    fail_unless(gap_range.begin()->lu() == 1);
-    fail_unless(gap_range.begin()->hs() == 1);
-    fail_unless(gap_range.rbegin()->lu() == 5);
-    fail_unless(gap_range.rbegin()->hs() == 7);
+    ck_assert(gap_range.size() == 2);
+    ck_assert(gap_range.begin()->lu() == 1);
+    ck_assert(gap_range.begin()->hs() == 1);
+    ck_assert(gap_range.rbegin()->lu() == 5);
+    ck_assert(gap_range.rbegin()->hs() == 7);
 
     im.insert(0, gcomm::evs::UserMessage(0, uuid, view_id, 1, 0));
     im.insert(0, gcomm::evs::UserMessage(0, uuid, view_id, 5, 0));
     im.insert(0, gcomm::evs::UserMessage(0, uuid, view_id, 6, 0));
     im.insert(0, gcomm::evs::UserMessage(0, uuid, view_id, 7, 0));
     gap_range = im.gap_range_list(0, gcomm::evs::Range(0, 8));
-    fail_unless(gap_range.empty());
+    ck_assert(gap_range.empty());
 }
 END_TEST
 
@@ -478,43 +480,43 @@ static void single_join(DummyTransport* t, Proto* p)
     p->send_join();
 
     Datagram* rb = get_msg(t, &jm);
-    fail_unless(rb != 0);
-    fail_unless(jm.type() == Message::EVS_T_JOIN);
+    ck_assert(rb != 0);
+    ck_assert(jm.type() == Message::EVS_T_JOIN);
 
     // Install message is emitted at the end of JOIN handling
     // 'cause this is the only instance and is always consistent
     // with itself
     rb = get_msg(t, &im);
-    fail_unless(rb != 0);
-    fail_unless(im.type() == Message::EVS_T_INSTALL);
+    ck_assert(rb != 0);
+    ck_assert(im.type() == Message::EVS_T_INSTALL);
 
     // Handling INSTALL message emits three gap messages,
     // one for receiving install message (commit gap), one for
     // shift to install and one for shift to operational
     rb = get_msg(t, &gm);
-    fail_unless(rb != 0);
-    fail_unless(gm.type() == Message::EVS_T_GAP);
-    fail_unless((gm.flags() & Message::F_COMMIT) != 0);
+    ck_assert(rb != 0);
+    ck_assert(gm.type() == Message::EVS_T_GAP);
+    ck_assert((gm.flags() & Message::F_COMMIT) != 0);
 
     rb = get_msg(t, &gm);
-    fail_unless(rb != 0);
-    fail_unless(gm.type() == Message::EVS_T_GAP);
-    fail_unless((gm.flags() & Message::F_COMMIT) == 0);
+    ck_assert(rb != 0);
+    ck_assert(gm.type() == Message::EVS_T_GAP);
+    ck_assert((gm.flags() & Message::F_COMMIT) == 0);
 
     rb = get_msg(t, &gm);
-    fail_unless(rb != 0);
-    fail_unless(gm.type() == Message::EVS_T_GAP);
-    fail_unless((gm.flags() & Message::F_COMMIT) == 0);
+    ck_assert(rb != 0);
+    ck_assert(gm.type() == Message::EVS_T_GAP);
+    ck_assert((gm.flags() & Message::F_COMMIT) == 0);
 
     // State must have evolved JOIN -> S_GATHER -> S_INSTALL -> S_OPERATIONAL
-    fail_unless(p->state() == Proto::S_OPERATIONAL);
+    ck_assert(p->state() == Proto::S_OPERATIONAL);
 
     // Handle join message again, must stay in S_OPERATIONAL, must not
     // emit anything
     p->handle_msg(jm);
     rb = get_msg(t, &gm);
-    fail_unless(rb == 0);
-    fail_unless(p->state() == Proto::S_OPERATIONAL);
+    ck_assert(rb == 0);
+    ck_assert(p->state() == Proto::S_OPERATIONAL);
 
 }
 
@@ -561,107 +563,107 @@ static void double_join(DummyTransport* t1, Proto* p1,
 
     // Initial states check
     p2->shift_to(Proto::S_JOINING);
-    fail_unless(p1->state() == Proto::S_OPERATIONAL);
-    fail_unless(p2->state() == Proto::S_JOINING);
+    ck_assert(p1->state() == Proto::S_OPERATIONAL);
+    ck_assert(p2->state() == Proto::S_JOINING);
 
     // Send join message, don't self handle immediately
     // Expected output: one join message
     p2->send_join(false);
-    fail_unless(p2->state() == Proto::S_JOINING);
+    ck_assert(p2->state() == Proto::S_JOINING);
     rb = get_msg(t2, &jm);
-    fail_unless(rb != 0);
-    fail_unless(jm.type() == Message::EVS_T_JOIN);
+    ck_assert(rb != 0);
+    ck_assert(jm.type() == Message::EVS_T_JOIN);
     rb = get_msg(t2, &msg);
-    fail_unless(rb == 0);
+    ck_assert(rb == 0);
 
     // Handle node 2's join on node 1
     // Expected output: shift to S_GATHER and one join message
     p1->handle_msg(jm);
-    fail_unless(p1->state() == Proto::S_GATHER);
+    ck_assert(p1->state() == Proto::S_GATHER);
     rb = get_msg(t1, &jm);
-    fail_unless(rb != 0);
-    fail_unless(jm.type() == Message::EVS_T_JOIN);
+    ck_assert(rb != 0);
+    ck_assert(jm.type() == Message::EVS_T_JOIN);
     rb = get_msg(t1, &msg);
-    fail_unless(rb == 0);
+    ck_assert(rb == 0);
 
     // Handle node 1's join on node 2
     // Expected output: shift to S_GATHER and one join message
     p2->handle_msg(jm);
-    fail_unless(p2->state() == Proto::S_GATHER);
+    ck_assert(p2->state() == Proto::S_GATHER);
     rb = get_msg(t2, &jm);
-    fail_unless(rb != 0);
-    fail_unless(jm.type() == Message::EVS_T_JOIN);
+    ck_assert(rb != 0);
+    ck_assert(jm.type() == Message::EVS_T_JOIN);
     rb = get_msg(t2, &msg);
-    fail_unless(rb == 0);
+    ck_assert(rb == 0);
 
     // Handle node 2's join on node 1
     // Expected output: Install and commit gap messages, state stays in S_GATHER
     p1->handle_msg(jm);
-    fail_unless(p1->state() == Proto::S_GATHER);
+    ck_assert(p1->state() == Proto::S_GATHER);
     rb = get_msg(t1, &im);
-    fail_unless(rb != 0);
-    fail_unless(im.type() == Message::EVS_T_INSTALL);
+    ck_assert(rb != 0);
+    ck_assert(im.type() == Message::EVS_T_INSTALL);
     rb = get_msg(t1, &gm);
-    fail_unless(rb != 0);
-    fail_unless(gm.type() == Message::EVS_T_GAP);
-    fail_unless((gm.flags() & Message::F_COMMIT) != 0);
+    ck_assert(rb != 0);
+    ck_assert(gm.type() == Message::EVS_T_GAP);
+    ck_assert((gm.flags() & Message::F_COMMIT) != 0);
     rb = get_msg(t1, &msg);
-    fail_unless(rb == 0);
+    ck_assert(rb == 0);
 
     // Handle install message on node 2
     // Expected output: commit gap message and state stays in S_RECOVERY
     p2->handle_msg(im);
-    fail_unless(p2->state() == Proto::S_GATHER);
+    ck_assert(p2->state() == Proto::S_GATHER);
     rb = get_msg(t2, &gm2);
-    fail_unless(rb != 0);
-    fail_unless(gm2.type() == Message::EVS_T_GAP);
-    fail_unless((gm2.flags() & Message::F_COMMIT) != 0);
+    ck_assert(rb != 0);
+    ck_assert(gm2.type() == Message::EVS_T_GAP);
+    ck_assert((gm2.flags() & Message::F_COMMIT) != 0);
     rb = get_msg(t2, &msg);
-    fail_unless(rb == 0);
+    ck_assert(rb == 0);
 
     // Handle gap messages
     // Expected output: Both nodes shift to S_INSTALL,
     // both send gap messages
     p1->handle_msg(gm2);
-    fail_unless(p1->state() == Proto::S_INSTALL);
+    ck_assert(p1->state() == Proto::S_INSTALL);
     Message gm12;
     rb = get_msg(t1, &gm12);
-    fail_unless(rb != 0);
-    fail_unless(gm12.type() == Message::EVS_T_GAP);
-    fail_unless((gm12.flags() & Message::F_COMMIT) == 0);
+    ck_assert(rb != 0);
+    ck_assert(gm12.type() == Message::EVS_T_GAP);
+    ck_assert((gm12.flags() & Message::F_COMMIT) == 0);
     rb = get_msg(t1, &msg);
-    fail_unless(rb == 0);
+    ck_assert(rb == 0);
 
     p2->handle_msg(gm);
-    fail_unless(p2->state() == Proto::S_INSTALL);
+    ck_assert(p2->state() == Proto::S_INSTALL);
     Message gm22;
     rb = get_msg(t2, &gm22);
-    fail_unless(rb != 0);
-    fail_unless(gm22.type() == Message::EVS_T_GAP);
-    fail_unless((gm22.flags() & Message::F_COMMIT) == 0);
+    ck_assert(rb != 0);
+    ck_assert(gm22.type() == Message::EVS_T_GAP);
+    ck_assert((gm22.flags() & Message::F_COMMIT) == 0);
     rb = get_msg(t2, &msg);
-    fail_unless(rb == 0);
+    ck_assert(rb == 0);
 
     // Handle final gap messages, expected output shift to operational
     // and gap message
 
     p1->handle_msg(gm22);
-    fail_unless(p1->state() == Proto::S_OPERATIONAL);
+    ck_assert(p1->state() == Proto::S_OPERATIONAL);
     rb = get_msg(t1, &msg);
-    fail_unless(rb != 0);
-    fail_unless(msg.type() == Message::EVS_T_GAP);
-    fail_unless((msg.flags() & Message::F_COMMIT) == 0);
+    ck_assert(rb != 0);
+    ck_assert(msg.type() == Message::EVS_T_GAP);
+    ck_assert((msg.flags() & Message::F_COMMIT) == 0);
     rb = get_msg(t1, &msg);
-    fail_unless(rb == 0);
+    ck_assert(rb == 0);
 
     p2->handle_msg(gm12);
-    fail_unless(p2->state() == Proto::S_OPERATIONAL);
+    ck_assert(p2->state() == Proto::S_OPERATIONAL);
     rb = get_msg(t2, &msg);
-    fail_unless(rb != 0);
-    fail_unless(msg.type() == Message::EVS_T_GAP);
-    fail_unless((msg.flags() & Message::F_COMMIT) == 0);
+    ck_assert(rb != 0);
+    ck_assert(msg.type() == Message::EVS_T_GAP);
+    ck_assert((msg.flags() & Message::F_COMMIT) == 0);
     rb = get_msg(t2, &msg);
-    fail_unless(rb == 0);
+    ck_assert(rb == 0);
 
 }
 
@@ -1682,15 +1684,15 @@ START_TEST(test_trac_724)
     Proto* evs0(evs_from_dummy(dn[0]));
 
     bool ret(evs0->set_param("evs.use_aggregate", "false", sync_param_cb));
-    fail_unless(ret == true);
+    ck_assert(ret == true);
     ret = evs0->set_param("evs.send_window", "1024", sync_param_cb);
-    fail_unless(ret == true);
+    ck_assert(ret == true);
     ret = evs0->set_param("evs.user_send_window", "515", sync_param_cb);
     Proto* evs1(evs_from_dummy(dn[1]));
     ret = evs1->set_param("evs.use_aggregate", "false", sync_param_cb);
-    fail_unless(ret == true);
+    ck_assert(ret == true);
     ret = evs1->set_param("evs.send_window", "1024", sync_param_cb);
-    fail_unless(ret == true);
+    ck_assert(ret == true);
     ret = evs1->set_param("evs.user_send_window", "512", sync_param_cb);
 
     prop.set_loss(1, 2, 0.);
@@ -1938,14 +1940,14 @@ START_TEST(test_gh_40)
 
     Proto* evs1 = evs_from_dummy(dn[1]);
     Proto* evs2 = evs_from_dummy(dn[2]);
-    fail_if(evs1->state() != Proto::S_OPERATIONAL);
-    fail_if(evs2->state() != Proto::S_OPERATIONAL);
+    ck_assert(evs1->state() == Proto::S_OPERATIONAL);
+    ck_assert(evs2->state() == Proto::S_OPERATIONAL);
     evs1->set_inactive(dn[0]->uuid());
     evs2->set_inactive(dn[0]->uuid());
     evs1->check_inactive();
     evs2->check_inactive();
-    fail_if(evs1->state() != Proto::S_GATHER);
-    fail_if(evs2->state() != Proto::S_GATHER);
+    ck_assert(evs1->state() == Proto::S_GATHER);
+    ck_assert(evs2->state() == Proto::S_GATHER);
 
     // Advance clock to get over join message rate limiting.
     gu::datetime::SimClock::inc_time(100*gu::datetime::MSec);
@@ -2040,66 +2042,67 @@ START_TEST(test_gh_100)
 
     // Initial states check
     p2.shift_to(Proto::S_JOINING);
-    fail_unless(p1.state() == Proto::S_OPERATIONAL);
-    fail_unless(p2.state() == Proto::S_JOINING);
+    ck_assert(p1.state() == Proto::S_OPERATIONAL);
+    ck_assert(p2.state() == Proto::S_JOINING);
 
     // Send join message, don't self handle immediately
     // Expected output: one join message
     p2.send_join(false);
-    fail_unless(p2.state() == Proto::S_JOINING);
+    ck_assert(p2.state() == Proto::S_JOINING);
     rb = get_msg(&t2, &jm);
-    fail_unless(rb != 0);
-    fail_unless(jm.type() == Message::EVS_T_JOIN);
+    ck_assert(rb != 0);
+    ck_assert(jm.type() == Message::EVS_T_JOIN);
     rb = get_msg(&t2, &msg);
-    fail_unless(rb == 0);
+    ck_assert(rb == 0);
 
     // Handle node 2's join on node 1
     // Expected output: shift to S_GATHER and one join message
     p1.handle_msg(jm);
-    fail_unless(p1.state() == Proto::S_GATHER);
+    ck_assert(p1.state() == Proto::S_GATHER);
     rb = get_msg(&t1, &jm);
-    fail_unless(rb != 0);
-    fail_unless(jm.type() == Message::EVS_T_JOIN);
+    ck_assert(rb != 0);
+    ck_assert(jm.type() == Message::EVS_T_JOIN);
     rb = get_msg(&t1, &msg);
-    fail_unless(rb == 0);
+    ck_assert(rb == 0);
 
     // Handle node 1's join on node 2
     // Expected output: shift to S_GATHER and one join message
     p2.handle_msg(jm);
-    fail_unless(p2.state() == Proto::S_GATHER);
+    ck_assert(p2.state() == Proto::S_GATHER);
     rb = get_msg(&t2, &jm);
-    fail_unless(rb != 0);
-    fail_unless(jm.type() == Message::EVS_T_JOIN);
+    ck_assert(rb != 0);
+    ck_assert(jm.type() == Message::EVS_T_JOIN);
     rb = get_msg(&t2, &msg);
-    fail_unless(rb == 0);
+    ck_assert(rb == 0);
 
     // Handle node 2's join on node 1
     // Expected output: Install and commit gap messages, state stays in S_GATHER
     p1.handle_msg(jm);
-    fail_unless(p1.state() == Proto::S_GATHER);
+    ck_assert(p1.state() == Proto::S_GATHER);
     rb = get_msg(&t1, &im);
-    fail_unless(rb != 0);
-    fail_unless(im.type() == Message::EVS_T_INSTALL);
+    ck_assert(rb != 0);
+    ck_assert(im.type() == Message::EVS_T_INSTALL);
     rb = get_msg(&t1, &gm);
-    fail_unless(rb != 0);
-    fail_unless(gm.type() == Message::EVS_T_GAP);
-    fail_unless((gm.flags() & Message::F_COMMIT) != 0);
+    ck_assert(rb != 0);
+    ck_assert(gm.type() == Message::EVS_T_GAP);
+    ck_assert((gm.flags() & Message::F_COMMIT) != 0);
     rb = get_msg(&t1, &msg);
-    fail_unless(rb == 0);
-
-    // usleep(1100000);
+    ck_assert(rb == 0);
 
     // Handle timers to  to generate shift to GATHER
     p1.handle_inactivity_timer();
     p1.handle_install_timer();
     rb = get_msg(&t1, &jm);
-    fail_unless(rb != 0);
-    fail_unless(jm.type() == Message::EVS_T_JOIN);
+    ck_assert(rb != 0);
+    ck_assert(jm.type() == Message::EVS_T_JOIN);
     rb = get_msg(&t1, &im2);
-    fail_unless(rb != 0);
-    fail_unless(im2.type() == Message::EVS_T_INSTALL);
-    fail_unless(im2.install_view_id().seq() > im.install_view_id().seq());
+    ck_assert(rb != 0);
+    ck_assert(im2.type() == Message::EVS_T_INSTALL);
+    ck_assert(im2.install_view_id().seq() > im.install_view_id().seq());
 
+    gcomm::Datagram* tmp;
+    while ((tmp = t1.out())) delete tmp;
+    while ((tmp = t2.out())) delete tmp;
 }
 END_TEST
 
@@ -2119,7 +2122,7 @@ START_TEST(test_evs_protocol_upgrade)
         ++view_seq;
         for (int j(0); j <= i; ++j)
         {
-            fail_unless(evs_from_dummy(dn[j])->current_view().version() == 0);
+            ck_assert(evs_from_dummy(dn[j])->current_view().version() == 0);
             gu_trace(send_n(dn[j], 5 + ::rand() % 4));
         }
     }
@@ -2141,7 +2144,7 @@ START_TEST(test_evs_protocol_upgrade)
         }
         gu_trace(prop.propagate_until_empty());
     }
-    fail_unless(evs_from_dummy(dn[GCOMM_PROTOCOL_MAX_VERSION])->current_view().version() == GCOMM_PROTOCOL_MAX_VERSION);
+    ck_assert(evs_from_dummy(dn[GCOMM_PROTOCOL_MAX_VERSION])->current_view().version() == GCOMM_PROTOCOL_MAX_VERSION);
     check_trace(dn);
     for_each(dn.begin(), dn.end(), DeleteObject());
 }
@@ -2172,8 +2175,8 @@ START_TEST(test_gal_521)
     single_join(t1, evs1);
     double_join(t1, evs1, t2, evs2);
 
-    fail_unless(t1->empty() == true);
-    fail_unless(t2->empty() == true);
+    ck_assert(t1->empty() == true);
+    ck_assert(t2->empty() == true);
 
     // Adjust send windows to allow sending only single user generated
     // message at the time
@@ -2187,20 +2190,20 @@ START_TEST(test_gal_521)
     // each other. This will place one user message into transport
     // queue and one into send queue for both nodes.
     send_n(dn[0], 2);
-    fail_unless(t1->empty() == false);
+    ck_assert(t1->empty() == false);
     send_n(dn[1], 2);
-    fail_unless(t2->empty() == false);
+    ck_assert(t2->empty() == false);
 
     Datagram *d1;
     Message um1;
-    fail_unless((d1 = get_msg(t1, &um1, false)) != 0);
-    fail_unless(um1.type() == Message::EVS_T_USER);
-    fail_unless(t1->empty() == true);
+    ck_assert((d1 = get_msg(t1, &um1, false)) != 0);
+    ck_assert(um1.type() == Message::EVS_T_USER);
+    ck_assert(t1->empty() == true);
     Datagram *d2;
     Message um2;
-    fail_unless((d2 = get_msg(t2, &um2, false)) != 0);
-    fail_unless(um2.type() == Message::EVS_T_USER);
-    fail_unless(t2->empty() == true);
+    ck_assert((d2 = get_msg(t2, &um2, false)) != 0);
+    ck_assert(um2.type() == Message::EVS_T_USER);
+    ck_assert(t2->empty() == true);
 
     // Both of the nodes handle each other's messages. Now due to
     // send_window == 1 they are not allowed to send the second
@@ -2209,52 +2212,52 @@ START_TEST(test_gal_521)
     evs1->handle_up(0, *d2, ProtoUpMeta(dn[1]->uuid()));
     delete d2;
     Message gm1;
-    fail_unless(get_msg(t1, &gm1) != 0);
-    fail_unless(gm1.type() == Message::EVS_T_GAP);
-    fail_unless(t1->empty() == true);
+    ck_assert(get_msg(t1, &gm1) != 0);
+    ck_assert(gm1.type() == Message::EVS_T_GAP);
+    ck_assert(t1->empty() == true);
 
     evs2->handle_up(0, *d1, ProtoUpMeta(dn[0]->uuid()));
     delete d1;
     Message gm2;
-    fail_unless(get_msg(t2, &gm2) != 0);
-    fail_unless(gm2.type() == Message::EVS_T_GAP);
-    fail_unless(t2->empty() == true);
+    ck_assert(get_msg(t2, &gm2) != 0);
+    ck_assert(gm2.type() == Message::EVS_T_GAP);
+    ck_assert(t2->empty() == true);
 
     // Handle gap messages. The safe_seq is now incremented so the
     // second user messages are now sent from output queue.
     evs1->handle_msg(gm2);
-    fail_unless((d1 = get_msg(t1, &um1, false)) != 0);
-    fail_unless(um1.type() == Message::EVS_T_USER);
-    fail_unless(t1->empty() == true);
+    ck_assert((d1 = get_msg(t1, &um1, false)) != 0);
+    ck_assert(um1.type() == Message::EVS_T_USER);
+    ck_assert(t1->empty() == true);
 
     evs2->handle_msg(gm1);
-    fail_unless((d2 = get_msg(t2, &um2, false)) != 0);
-    fail_unless(um2.type() == Message::EVS_T_USER);
-    fail_unless(t2->empty() == true);
+    ck_assert((d2 = get_msg(t2, &um2, false)) != 0);
+    ck_assert(um2.type() == Message::EVS_T_USER);
+    ck_assert(t2->empty() == true);
 
     // Handle user messages. Each node should now emit gap
     // because the output queue is empty.
     evs1->handle_up(0, *d2, ProtoUpMeta(dn[1]->uuid()));
     delete d2;
-    fail_unless(get_msg(t1, &gm1) != 0);
-    fail_unless(gm1.type() == Message::EVS_T_GAP);
-    fail_unless(t1->empty() == true);
+    ck_assert(get_msg(t1, &gm1) != 0);
+    ck_assert(gm1.type() == Message::EVS_T_GAP);
+    ck_assert(t1->empty() == true);
 
     evs2->handle_up(0, *d1, ProtoUpMeta(dn[0]->uuid()));
     delete d1;
-    fail_unless(get_msg(t2, &gm2) != 0);
-    fail_unless(gm2.type() == Message::EVS_T_GAP);
-    fail_unless(t2->empty() == true);
+    ck_assert(get_msg(t2, &gm2) != 0);
+    ck_assert(gm2.type() == Message::EVS_T_GAP);
+    ck_assert(t2->empty() == true);
 
     // Handle gap messages. No further messages should be emitted
     // since both user messages have been delivered, there are
     // no pending user messages in the output queue and no timers
     // have been expired.
     evs1->handle_msg(gm2);
-    fail_unless((d1 = get_msg(t1, &um1, false)) == 0);
+    ck_assert((d1 = get_msg(t1, &um1, false)) == 0);
 
     evs2->handle_msg(gm1);
-    fail_unless((d2 = get_msg(t2, &um2, false)) == 0);
+    ck_assert((d2 = get_msg(t2, &um2, false)) == 0);
 
 
     std::for_each(dn.begin(), dn.end(), DeleteObject());
@@ -2331,52 +2334,52 @@ START_TEST(test_gap_rate_limit)
     gcomm::Datagram* read_dg;
     gcomm::evs::Message um1;
     read_dg = get_msg(&f.tr1, &um1);
-    fail_unless(read_dg != 0);
+    ck_assert(read_dg != 0);
     f.evs1.handle_down(dg, ProtoDownMeta(O_SAFE));
     gcomm::evs::Message um2;
     read_dg = get_msg(&f.tr1, &um2);
-    fail_unless(read_dg != 0);
+    ck_assert(read_dg != 0);
     f.evs1.handle_down(dg, ProtoDownMeta(O_SAFE));
     gcomm::evs::Message um3;
     read_dg = get_msg(&f.tr1, &um3);
-    fail_unless(read_dg != 0);
+    ck_assert(read_dg != 0);
     f.evs1.handle_down(dg, ProtoDownMeta(O_SAFE));
     gcomm::evs::Message um4;
     read_dg = get_msg(&f.tr1, &um4);
-    fail_unless(read_dg != 0);
+    ck_assert(read_dg != 0);
 
     // Make node2 handle an out of order message and verify that gap is emitted
     f.evs2.handle_msg(um2);
     gcomm::evs::Message gm1;
     read_dg = get_msg(&f.tr2, &gm1);
-    fail_unless(read_dg != 0);
-    fail_unless(gm1.type() == gcomm::evs::Message::EVS_T_GAP);
-    fail_unless(gm1.range_uuid() == f.uuid1);
-    fail_unless(gm1.range().lu() == 0);
-    fail_unless(gm1.range().hs() == 0);
+    ck_assert(read_dg != 0);
+    ck_assert(gm1.type() == gcomm::evs::Message::EVS_T_GAP);
+    ck_assert(gm1.range_uuid() == f.uuid1);
+    ck_assert(gm1.range().lu() == 0);
+    ck_assert(gm1.range().hs() == 0);
     // The node2 will also send an user message to complete the sequence
     // number. Consume it.
     gcomm::evs::Message comp_um1;
     read_dg = get_msg(&f.tr2, &comp_um1);
-    fail_unless(read_dg != 0);
-    fail_unless(comp_um1.type() == gcomm::evs::Message::EVS_T_USER);
-    fail_unless(comp_um1.seq() + comp_um1.seq_range() == 1);
+    ck_assert(read_dg != 0);
+    ck_assert(comp_um1.type() == gcomm::evs::Message::EVS_T_USER);
+    ck_assert(comp_um1.seq() + comp_um1.seq_range() == 1);
     // No further messages should be emitted
     read_dg = get_msg(&f.tr2, &comp_um1);
-    fail_if(read_dg != 0);
+    ck_assert(read_dg == 0);
 
     // Handle the second out of order message, gap should not be emitted.
     // There will be a next user message which completes the um3.
     f.evs2.handle_msg(um3);
     gcomm::evs::Message comp_um2;
     read_dg = get_msg(&f.tr2, &comp_um2);
-    fail_unless(read_dg != 0);
-    fail_unless(comp_um2.type() == gcomm::evs::Message::EVS_T_USER);
-    fail_unless(comp_um2.seq() + comp_um2.seq_range() == 2);
+    ck_assert(read_dg != 0);
+    ck_assert(comp_um2.type() == gcomm::evs::Message::EVS_T_USER);
+    ck_assert(comp_um2.seq() + comp_um2.seq_range() == 2);
 
     // There should not be any more gap messages.
     read_dg = get_msg(&f.tr2, &gm1);
-    fail_if(read_dg != 0);
+    ck_assert(read_dg == 0);
 
     // Move the clock forwards and handle the fourth message, gap should
     // now emitted.
@@ -2384,15 +2387,15 @@ START_TEST(test_gap_rate_limit)
     gcomm::evs::Message gm2;
     f.evs2.handle_msg(um4);
     read_dg = get_msg(&f.tr2, &gm2);
-    fail_unless(read_dg != 0);
-    fail_unless(gm2.type() == gcomm::evs::Message::EVS_T_GAP);
-    fail_unless(gm2.range().lu() == 0);
-    fail_unless(gm2.range().hs() == 0);
+    ck_assert(read_dg != 0);
+    ck_assert(gm2.type() == gcomm::evs::Message::EVS_T_GAP);
+    ck_assert(gm2.range().lu() == 0);
+    ck_assert(gm2.range().hs() == 0);
 
     gcomm::evs::Message comp_u4;
     read_dg = get_msg(&f.tr2, &comp_u4);
-    fail_unless(read_dg != 0);
-    fail_unless(comp_u4.type() == gcomm::evs::Message::EVS_T_USER);
+    ck_assert(read_dg != 0);
+    ck_assert(comp_u4.type() == gcomm::evs::Message::EVS_T_USER);
     log_info << "END test_gap_rate_limit";
 }
 END_TEST
@@ -2426,39 +2429,39 @@ START_TEST(test_gap_rate_limit_delayed)
     gcomm::Datagram* read_dg;
     gcomm::evs::Message um1;
     read_dg = get_msg(&f.tr1, &um1);
-    fail_unless(read_dg != 0);
+    ck_assert(read_dg != 0);
     f.evs1.handle_down(dg, ProtoDownMeta(O_SAFE));
     gcomm::evs::Message um2;
     read_dg = get_msg(&f.tr1, &um2);
-    fail_unless(read_dg != 0);
+    ck_assert(read_dg != 0);
     f.evs1.handle_down(dg, ProtoDownMeta(O_SAFE));
     gcomm::evs::Message um3;
     read_dg = get_msg(&f.tr1, &um3);
-    fail_unless(read_dg != 0);
+    ck_assert(read_dg != 0);
     f.evs1.handle_down(dg, ProtoDownMeta(O_SAFE));
     gcomm::evs::Message um4;
     read_dg = get_msg(&f.tr1, &um4);
-    fail_unless(read_dg != 0);
+    ck_assert(read_dg != 0);
 
     // Make node2 handle an out of order message and verify that gap is emitted
     f.evs2.handle_msg(um2);
     gcomm::evs::Message gm1;
     read_dg = get_msg(&f.tr2, &gm1);
-    fail_unless(read_dg != 0);
-    fail_unless(gm1.type() == gcomm::evs::Message::EVS_T_GAP);
-    fail_unless(gm1.range_uuid() == f.uuid1);
-    fail_unless(gm1.range().lu() == 0);
-    fail_unless(gm1.range().hs() == 0);
+    ck_assert(read_dg != 0);
+    ck_assert(gm1.type() == gcomm::evs::Message::EVS_T_GAP);
+    ck_assert(gm1.range_uuid() == f.uuid1);
+    ck_assert(gm1.range().lu() == 0);
+    ck_assert(gm1.range().hs() == 0);
     // The node2 will also send an user message to complete the sequence
     // number. Consume it.
     gcomm::evs::Message comp_um1;
     read_dg = get_msg(&f.tr2, &comp_um1);
-    fail_unless(read_dg != 0);
-    fail_unless(comp_um1.type() == gcomm::evs::Message::EVS_T_USER);
-    fail_unless(comp_um1.seq() + comp_um1.seq_range() == 1);
+    ck_assert(read_dg != 0);
+    ck_assert(comp_um1.type() == gcomm::evs::Message::EVS_T_USER);
+    ck_assert(comp_um1.seq() + comp_um1.seq_range() == 1);
     // No further messages should be emitted
     read_dg = get_msg(&f.tr2, &comp_um1);
-    fail_if(read_dg != 0);
+    ck_assert(read_dg == 0);
 
     // Move time forwards in 1 sec interval and make inactivity check
     // in between. No gap messages should be emitted.
@@ -2466,15 +2469,15 @@ START_TEST(test_gap_rate_limit_delayed)
     f.evs2.handle_inactivity_timer();
     gcomm::evs::Message gm_discard;
     read_dg = get_msg(&f.tr2, &gm_discard);
-    fail_if(read_dg != 0);
+    ck_assert(read_dg == 0);
     // The clock is now advanced over retrans_period + delay margin. Next
     // call to handle_inactivity_timer() should fire the check. Gap message
     // is emitted.
     gu::datetime::SimClock::inc_time(gu::datetime::Sec);
     f.evs2.handle_inactivity_timer();
     read_dg = get_msg(&f.tr2, &gm1);
-    fail_unless(read_dg != 0);
-    fail_unless(gm1.type() == gcomm::evs::Message::EVS_T_GAP);
+    ck_assert(read_dg != 0);
+    ck_assert(gm1.type() == gcomm::evs::Message::EVS_T_GAP);
     // Now call handle_inactivity_timer() again, gap message should not
     // be emitted due to rate limit.
     // Galera 4 will run with evs protocol version 1 and will emit
@@ -2482,18 +2485,22 @@ START_TEST(test_gap_rate_limit_delayed)
     f.evs2.handle_inactivity_timer();
     gcomm::evs::Message dm;
     read_dg = get_msg(&f.tr2, &dm);
-    fail_unless(read_dg != 0);
-    fail_unless(dm.type() == gcomm::evs::Message::EVS_T_DELAYED_LIST);
+    ck_assert(read_dg != 0);
+    ck_assert(dm.type() == gcomm::evs::Message::EVS_T_DELAYED_LIST);
     read_dg = get_msg(&f.tr2, &gm_discard);
-    fail_if(read_dg != 0);
+    ck_assert(read_dg == 0);
     // Move clock forward 100msec, new gap should be now emitted.
     gu::datetime::SimClock::inc_time(100*gu::datetime::MSec);
     f.evs2.handle_inactivity_timer();
     gcomm::evs::Message gm2;
     read_dg = get_msg(&f.tr2, &gm2);
-    fail_unless(read_dg != 0);
-    fail_unless(gm2.type() == gcomm::evs::Message::EVS_T_GAP);
+    ck_assert(read_dg != 0);
+    ck_assert(gm2.type() == gcomm::evs::Message::EVS_T_GAP);
     log_info << "END test_gap_rate_limit_delayed";
+
+    gcomm::Datagram* tmp;
+    while ((tmp = f.tr1.out())) delete tmp;
+    while ((tmp = f.tr2.out())) delete tmp;
 }
 END_TEST
 
@@ -2508,12 +2515,15 @@ START_TEST(test_out_queue_limit)
     // so we can write 2 + 32 messages without blocking.
     for (size_t i(0); i < 34; ++i)
     {
-        fail_unless(f.evs1.handle_down(dg, ProtoDownMeta(O_SAFE)) == 0);
+        ck_assert(f.evs1.handle_down(dg, ProtoDownMeta(O_SAFE)) == 0);
     }
     // The next write should fill the out_queue and return EAGAIN
     const char small_data[1] = { 0 };
     dg = gu::SharedBuffer(new gu::Buffer(small_data, small_data + 1));
-    fail_unless(f.evs1.handle_down(dg, ProtoDownMeta(O_SAFE)) == EAGAIN);
+    ck_assert(f.evs1.handle_down(dg, ProtoDownMeta(O_SAFE)) == EAGAIN);
+
+    gcomm::Datagram* tmp;
+    while ((tmp = f.tr1.out())) delete tmp;
 }
 END_TEST
 
@@ -2573,8 +2583,8 @@ START_TEST(test_representative_incarnation_change)
     gu::datetime::SimClock::inc_time(300 * gu::datetime::MSec);
     evs_from_dummy(dn[1])->handle_timers();
     evs_from_dummy(dn[2])->handle_timers();
-    fail_unless(evs_from_dummy(dn[1])->state() == gcomm::evs::Proto::S_GATHER);
-    fail_unless(evs_from_dummy(dn[2])->state() == gcomm::evs::Proto::S_GATHER);
+    ck_assert(evs_from_dummy(dn[1])->state() == gcomm::evs::Proto::S_GATHER);
+    ck_assert(evs_from_dummy(dn[2])->state() == gcomm::evs::Proto::S_GATHER);
 
     // Create a new instance with old representative uuid with incarnation
     // incremented. Keep it isolated from old representative.
