@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008-2020 Codership Oy <info@codership.com>
+ * Copyright (C) 2008-2021 Codership Oy <info@codership.com>
  *
  * $Id$
  */
@@ -713,7 +713,8 @@ _release_flow_control (gcs_conn_t* conn)
 static void
 gcs_become_primary (gcs_conn_t* conn)
 {
-    assert(conn->join_gtid.seqno() == GCS_SEQNO_ILL ||
+    assert(conn->join_gtid.seqno() <= 0      ||
+           conn->state == GCS_CONN_PRIMARY   ||
            conn->state == GCS_CONN_JOINER    ||
            conn->state == GCS_CONN_OPEN /* joiner that has received NON_PRIM */);
 
@@ -1682,7 +1683,7 @@ long gcs_destroy (gcs_conn_t *conn)
     gu_cond_t tmp_cond;
     gu_cond_init (&tmp_cond, NULL);
 
-    if ((err = gcs_sm_enter (conn->sm, &tmp_cond, false, true))) // need an error here
+    if (!(err = gcs_sm_enter (conn->sm, &tmp_cond, false, true))) // need an error here
     {
         if (GCS_CONN_CLOSED != conn->state)
         {
@@ -1695,20 +1696,19 @@ long gcs_destroy (gcs_conn_t *conn)
             return -EBADFD;
         }
 
-        /* this should cancel all recv calls */
-        gu_fifo_destroy (conn->recv_q);
+        gcs_sm_leave (conn->sm);
 
         gcs_shift_state (conn, GCS_CONN_DESTROYED);
         /* we must unlock the mutex here to allow unfortunate threads
          * to acquire the lock and give up gracefully */
     }
     else {
-        gcs_sm_leave (conn->sm);
-        gu_cond_destroy (&tmp_cond);
-        err = -EBADFD;
-        return err;
+        gu_debug("gcs_destroy: gcs_sm_enter() err = %d", err);
+        // We should still cleanup resources
     }
 
+    gu_fifo_destroy (conn->recv_q);
+
     gu_cond_destroy (&tmp_cond);
     gcs_sm_destroy (conn->sm);
 
@@ -1788,6 +1788,12 @@ long gcs_caused(gcs_conn_t* conn, gu::GTID& gtid)
     return gcs_core_caused(conn->core, gtid);
 }
 
+static inline bool
+fc_active(gcs_conn_t* conn)
+{
+    return conn->stop_count > 0;
+}
+
 /* Puts action in the send queue and returns after it is replicated */
 long gcs_replv (gcs_conn_t*          const conn,      //!<in
                 const struct gu_buf* const act_in,    //!<in
@@ -1829,9 +1835,8 @@ long gcs_replv (gcs_conn_t*          const conn,      //!<in
             // if (conn->state >= GCS_CONN_CLOSE) or (act_ptr == NULL)
             // ret will be -ENOTCONN
             if ((ret = -EAGAIN,
-                 conn->upper_limit >= conn->queue_len ||
-                 act->type         != GCS_ACT_WRITESET)         &&
-                (ret = -ENOTCONN, GCS_CONN_OPEN >= conn->state) &&
+                 !fc_active(conn) || act->type != GCS_ACT_WRITESET) &&
+                (ret = -ENOTCONN, GCS_CONN_OPEN >= conn->state)     &&
                 (act_ptr = (struct gcs_repl_act**)gcs_fifo_lite_get_tail (conn->repl_q)))
             {
                 *act_ptr = &repl_act;
@@ -2236,15 +2241,15 @@ gcs_vote (gcs_conn_t* const conn, const gu::GTID& gtid, uint64_t const code,
     if (0 != code)
     {
         size_t const buf_len(gtid.serial_size() + sizeof(code));
-        char* const buf(new char[buf_len]);
+        std::vector<char> buf(buf_len);
         size_t offset(0);
 
-        offset = gtid.serialize(buf, buf_len, offset);
-        offset = gu::serialize8(code, buf, buf_len, offset);
-        assert(buf_len == offset);
+        offset = gtid.serialize(buf.data(), buf.size(), offset);
+        offset = gu::serialize8(code, buf.data(), buf.size(), offset);
+        assert(buf.size() == offset);
 
         gu::MMH3 hash;
-        hash.append(buf, buf_len);
+        hash.append(buf.data(), buf.size());
         hash.append(msg, msg_len);
 
         my_vote = (hash.gather8() | (1ULL << 63));
@@ -2326,6 +2331,8 @@ gcs_get_stats (gcs_conn_t* conn, struct gcs_stats* stats)
     stats->fc_ssent    = conn->stats_fc_stop_sent;
     stats->fc_csent    = conn->stats_fc_cont_sent;
     stats->fc_received = conn->stats_fc_received;
+    stats->fc_active   = fc_active(conn);
+    stats->fc_requested= conn->stop_sent_ > 0;
 }
 
 void
