@@ -93,9 +93,8 @@ elif [ "$OS" == "FreeBSD" ]; then
 fi
 
 if [ "$OS" == "FreeBSD" ]; then
-    CC=${CC:-"gcc48"}
-    CXX=${CXX:-"g++48"}
-    LD_LIBRARY_PATH=${LD_LIBRARY_PATH:-"/usr/local/lib/$(basename $CC)"}
+    CC=${CC:-"clang"}
+    CXX=${CXX:-"clang++"}
 else
     CC=${CC:-"gcc"}
     CXX=${CXX:-"g++"}
